string.split = function(str, token)
    local result = {}
    string.gsub(str, '[^'..token..']+', function(word)
        table.insert(result, word)
    end)
    return result
end

local FlashInfo = class("FlashInfo", function(_, path, time, fadeInTime, fadeOutTime)
        local self = {}
        -- 图片路径
        self.path = path
        -- 播放停留时间
        self.time = time or 2
        self.fadeInTime = fadeInTime or 1
        self.fadeOutTime = fadeOutTime or 1
        return self
    end)


GameScene = class("GameScene", function(_, verCode) 
	require "src/UpdateView" 
    math.randomseed(os.time())
	local self = cc.Scene:create() 

	self.config = nil 
	self.updateView = nil
	self.updateMgr = nil
    self.sdkInterface = nil
    self.verCode = verCode
    self.json = require("json")

    self.flashes = { FlashInfo:new("flash.png", 2, 1, 1), FlashInfo:new("flash1.png", 2, 1, 1) }
    self.flashInfo = nil
    self.flashImg = nil  
    self.flashIdx = 1

	return self
end)

function GameScene:ctor( )
    if #self.flashes == 0 then
        self:start()
    else
        self:showNextFlash()
    end
end

function GameScene:showNextFlash()
    local function next()
        self:showNextFlash()
    end 

    if self.flashInfo ~= nil then 
        self:removeChild(self.flashImg)
        self.flashImg:stopAllActions()
        self.flashImg:release()
        self.flashImg = nil
        self.flashInfo = nil
    end

    if #self.flashes < self.flashIdx then
        self:start()
    else
        local size = cc.Director:getInstance():getVisibleSize()
        self.flashInfo = self.flashes[self.flashIdx]
        self.flashIdx = self.flashIdx + 1 
        if self.flashInfo ~= nil then
            local actionOut = cc.FadeOut:create(self.flashInfo.fadeOutTime)--淡出
            local actionIn = cc.FadeIn:create(self.flashInfo.fadeInTime) -- 渐入
            local action = cc.RotateBy:create(self.flashInfo.time,0) -- 旋转
            local callback = cc.CallFunc:create(next)

            local anima = cc.Sequence:create(actionIn, action, actionOut, callback) 

            self.flashImg = cc.Sprite:create(self.flashInfo.path)
            if self.flashImg == nil then
                self.flashInfo = nil
                self:showNextFlash()
            else
                self.flashImg:setOpacity(0)
                self.flashImg:retain()
                self.flashImg:setAnchorPoint( cc.p( 0.5, 0.5 ) ) 
                self.flashImg:setPosition( size.width / 2, size.height / 2 )
                self:addChild(self.flashImg)
                self.flashImg:runAction(anima) 
            end
        end
    end
 
end

function GameScene:start()
	local platformId = cc.Application:getInstance():getTargetPlatform() 
    self.config = GameConfig:new(platformId)
    self.config.bigVersion = self.verCode
    print("#self.config.bigVersion", self.config.bigVersion)

    local en = nil
    local className = ""
    local sdkId = 0
    if self.config:isIOS() then
        en = require("luaoc")
        className = "KYPlatform"
        local ok,res = en.callStaticMethod(className, "getSDKID")
        if(ok)then
            sdkId = tonumber(res.data)
        end
    elseif self.config:isAndroid() then
        en = require("luaj") 
        className = "org.cocos2dx.lua.Cocos2dxActivity"
        local ok,res = en.callStaticMethod(className,"getSDKID")
        if(ok)then
            sdkId = tonumber(res) 
        end
    end

    self.config:setSdkId(sdkId)
    if(self.config.sdkId == 0 or self.config.sdkId == 9999999)then
        en = nil
    end

    print("#正在为您启动游戏(*):", "(空):正在初始化，(!):初始化sdk完成")
    print("#(.):正在获取url，(!.):获取url成功")
    print("#(..):正在获取version.txt，(!..):获取version.txt成功")
    print("#(...):准备开始更新")

    self._startBg = cc.Sprite:create("res/GameUI"..self.config.trunkType.."/outPic/login_loadingBg.png")
    self.updateView = UpdateView:new(self._startBg,self.config)
    self.updateView:setProgress(100)
    self.updateView:setTips("正在为您启动游戏")
    self:addChild(self.updateView)

    self.flashIdx = 1

    sdkId = sdkId or 0
    print("sdkId:--------",sdkId)
    if(tonumber(sdkId) == 0 or tonumber(self.config.sdkId) == 9999999)then
        self.sdkInterface = BaseInterface:new(self,en,className) 
        -- self:enterGame()

        self.updateMgr = GameUpdate:new(self)
        self:startLuaUpdate()
    else
        -- 现在先直接测试更新
        self.updateMgr = GameUpdate:new(self)
        --初始化sdkInterface
        self.sdkInterface = BaseInterface:new(self,en,className)
        -- 先初始化SDK, 调用初始化, 再更新
        self.sdkInterface:setCallback()
        self.sdkInterface:luaCall({self.sdkInterface.initSdk})
    end
end

function GameScene:startLuaUpdate()
    self.updateMgr:checkUpdate()
end

function GameScene:enterGame()
	print("#GameScene:enterGame")
    require "src/luaGame"
    require "src/modules.lua"
    -- require "src/require2.lua"

    -- if(self.updateView ~= nil)then
    --     self.updateView:dispose()
    --     self.updateView = nil
    -- end

    local schedulerID = -1
    local tickCount = 0
    local function tick()
        tickCount = tickCount + 1
        if(tickCount <= 10)then
            return
        end
        local gameStart = GameStart:new()
        gameStart:setup(self)
        self:dispose()

        if(schedulerID ~= -1)then
            cc.Director:getInstance():getScheduler():unscheduleScriptEntry(schedulerID)
            schedulerID = -1
        end
    end

    schedulerID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(tick, 0, false)
end

function GameScene:clearUpdateView()
    if(self.updateView ~= nil)then
        self.updateView:dispose()
        self.updateView = nil
    end
end

function GameScene:dispose()

	self.config = nil 
	if self.updateMgr ~= nil then
		self.updateMgr:dispose()
		self.updateMgr = nil
	end
end

GameUpdate = class("GameUpdate", function(_, gameScene)
	local self = {}

	self.gameScene = gameScene
	self.config = gameScene.config
	self.json = gameScene.json
	self._updateView = gameScene.updateView
    self.loader = LoaderManager:create()

	self.isNeedRestart = false

    self.mgr = nil

    self._packKey = "PACKET_DOWNLOAD" 
    self._additionKey = "ADDITIONVERSION"
    self._updatePaths = {}
    self._additionPackets = {}
    self._updateCount = 0
    self._currentItem = nil
    self._tryTime = 300

    self.percentPast = -1

    self._hasError = 0
    self._downUrls = {}
    self._packFileName = "updateFile.zip"
    --根目录
    self._writablePath = cc.FileUtils:getInstance():getWritablePath()
    self._versionPath = self._writablePath..string.gsub(self.config.bigVersion, "%.", "_")
    print("#writablePath", self._writablePath)
    print("#versionPath", self._versionPath)

	return self
end)

function GameUpdate:ctor()
    -- local percentPast = -1
    print("### (.)准备更新，(..)正在下载，(...)正在解压缩，!Error(序号)，")

    local function redownload()
        if self._tryTime > 0 then
            self._tryTime = self._tryTime - 1
            self.config:writeKV(self._packKey, "" )
            table.insert(self._updatePaths, 1, self._currentItem)
            self.mgr:removeDownloadPacket() 
            self:updateNext()
        end
    end

    local function error(errorCode)
        print("#error", errorCode)
        if errorCode == 0 then
            --创建文件失败,直接提示
            -- self:showError(0)
            self:setNowProgress(self.percentPast, "!")
            redownload()
        elseif errorCode == 1 then
            --网络问题
            --提示，并且重新下载
            -- self:showError(1)
            self:setNowProgress(self.percentPast, "!.")
            redownload()
        elseif errorCode == 2 then
            --没有新版本
            self:setNowProgress(self.percentPast, "!..")
            self:updateComplete()
        elseif errorCode == 3 then
            --解压缩失败,重新下载
            self:setNowProgress(self.percentPast, "!...")
            redownload()
        end
    end

    local function progress(percent)
        self:setNowProgress(percent, "..")
    end

    --status: 1:全部完成, 2:全部开始, 3:单个开始, 4:单个结束, 5:解压开始, 6:解压结束
    local function success(status, val)  
        if status == 1 then
            self.percentPast = 1
        elseif status == 4 then
            local pVersion = self._currentItem.t .. self._currentItem.p
            --存储下载版本号?
            self.config:writeKV(self._packKey, pVersion )
            --开始解压
            self.mgr:uncompress()
        elseif status == 5 then
            self:setNowProgress(self.percentPast, "...")
        elseif status == 6 then 
            --删除下载的文件?
            self.mgr:removeDownloadPacket()
            if self._currentItem.t == "a" then
                self.config:writeKV(self._additionKey, self._currentItem.v)
                print("#.writeKV(self._additionKey, self._currentItem.v)", self._additionKey, self._currentItem.v )
            else
                self.config.resVersion = tonumber(self._currentItem.v)
            end

            self.config:writeKV(self._packKey, "" )  --清掉下载版本号
            --删除下载的版本号
            self._hasError = 0
            self:updateNext()
        end
    end
    self.mgr = AssetsUpdateMgr:create(error, progress, success)
    -- 设置下载包的文件名
    if self.mgr.setTempZipFile ~= nil then
        self.mgr:setTempZipFile(self._packFileName)
    end
    print( "#self.mgr.setWritablePath", self.mgr.setWritablePath )
    print( "#self.mgr.setTempZipFile", self.mgr.setTempZipFile )
    print( "#self.mgr.setUncompressFilePath", self.mgr.setUncompressFilePath )
end

function GameUpdate:setNowProgress( percent, addStr )

    addStr = addStr or ""
    local idx = self._updateCount - #self._updatePaths
    local count = self._updateCount
    if addStr == ".." then
        local tpercent = math.ceil((idx -1) / count * 100)
        percent = tpercent + math.ceil( percent * (100 / count) / 100 )
        percent = math.max(self.percentPast, percent)
        percent = math.max(0, math.min(100, percent))
        self._updateView:setProgress(percent)
    end
    local tipStr = "攻城狮们正在努力下载更新包"..self._currentItem.p.."("..idx.."/"..count.."),进度为"..percent.."%"..addStr
    -- print("#tipStr", tipStr)
    self._updateView:setTips(tipStr)
    self.percentPast = percent

end

function GameUpdate:checkUpdate()

    local function requestVersionCallback(datas)  
        local data = tolua.cast(datas, "LoaderResult")
        local vals = data:getStrData()
        print("#version.txt,", vals) 

        if vals == nil or vals == "" then
            print("#version.txt加载失败")
            loadVersion()
            do return end
        end
        self._updateView:setTips("正在为您启动游戏!..")

        vals = string.split(vals, "\r\n")

        --排序
        local codes = {}
        local passVer = self.config.resVersion
        local currentVer = 0
        local additionStr = "" 
        local validStr = "cdf"

        for i=1,#vals do
            local t = string.sub(vals[i], 1, 1) 
            if(t == "p")then
                self.config.shenHeBigVersion = {}
                string.gsub(vals[i], '[^,]+', function(word)
                    table.insert(self.config.shenHeBigVersion, string.sub(word, 2))
                end)
                print("#bigVersion:", self.config.bigVersion)
                print("#config Ver:", unpack(self.config.shenHeBigVersion))
            elseif(t == "c" or t == "d" or t == "f" or t == "u" or t == "g")then
                if self.config.sdkId ~= 0 then
                    currentVer = tonumber(string.sub(vals[i], 2))
                    if currentVer > passVer then
                        table.insert(codes, { v = currentVer, p = vals[i], t = t } )
                    end
                end

                local vs = ""
                string.gsub(validStr, '[^'..t..']+', function(word)
                    vs = vs .. word
                end)
                validStr = vs
            elseif(t == "v")then
                if self.config.sdkId ~= 0 then
                    local vers = string.split(vals[i], ",") 
                    for j = 1, #vers do
                        currentVer = tonumber(string.sub(vers[j], 2))
                        if currentVer > passVer then
                            table.insert(codes, { v = currentVer, p = vers[j], t = t } )
                        end
                    end
                end
            elseif(t == "b")then
                self.config.plusList = string.split(vals[i], ",")
                table.insert(self.config.plusList, 1, "b0")
            elseif(t == "a")then
                additionStr = vals[i]
            end
        end

        if string.len(validStr) > 0 then
            print("#验证不通过" .. validStr)
            loadVersion()
            do return end
        end

        function sortVer(a, b)
            return a.v < b.v
        end
        table.sort( codes, sortVer ) 

        -- 增量包处理
        local additionVersion = self.config:readKV(self._additionKey)
        passVer = 0
        if additionVersion ~= nil and additionVersion ~= "" then
            passVer = tonumber(additionVersion)
        end 
        local vers = string.split(additionStr, ",") 
        for j = #vers,1,-1 do
            currentVer = tonumber(string.sub(vers[j], 2))
            if currentVer > passVer then
                if self.config.sdkId == 99 then
                    table.insert(codes, 1, { v = currentVer, p = vers[j], t = "a", idx = j, count = #vers } )
                end
            end
        end

        local vlist, bigV = self.config.shenHeBigVersion, self.config.bigVersion
        for i=1,#vlist do
            if vlist[i] == bigV then
                codes = {}
            end
        end

        self._updatePaths = codes 
        self._updateCount = #self._updatePaths

        if self._updateCount > 0 then
            self._updateView:setTips("正在为您启动游戏...")
            print("#start update", #self._updatePaths) 
            self._hasError = 0
            self:updateNext()
        else
            print("#没有版本更新")
            self:updateComplete()
        end

    end

    function loadVersion()
        local url = self._downUrls[1]
        if self._hasError == 1 and self._downUrls[2] ~= nil then
            url = self._downUrls[2]
        end
        print("#version url", url)
        if url == nil then
            self:updateComplete()
        else
            print("#second request "..url)
            self._updateView:setTips("正在为您启动游戏..")
            -- self.config.verUrl = url
            self.loader:getRemote(url.."version.txt".."?"..tostring(math.random(1,100000)), requestVersionCallback)
        end
        self._hasError = self._hasError + 1
    end

    local function urlHandler(datas)  
        local data = tolua.cast(datas, "LoaderResult")
        local vals = data:getStrData() 
        self._updateView:setTips("正在为您启动游戏!.")
        print("#urlHandler", vals)
        vals = string.split(vals, "\r\n")

        --self.sdkId
        if vals ~= nil and #vals > 0 then
            local hasUrl = false 
            local dict = {}
            for i = 1, #vals do
                local item = string.split(vals[i], "###")
                print("#item", item[1], item[2], #item)
                if item ~= nil and #item == 2 then
                    dict[item[1]] = item[2]
                    hasUrl = true
                end
            end 
            if hasUrl == true then
                local key = tostring(self.config.sdkId)
                local url = dict[tostring(key)] 
                if url == nil then
                    key = "0"
                end 
                if(tonumber(self.config.sdkId) == 0 or tonumber(self.config.sdkId) == 9999999)then
                    key = "-1"
                end
                url = dict[key]
                table.insert(self._downUrls, url)
                local newUrl = dict[tostring(tonumber(key) + 20000)]
                if newUrl ~= nil then
                    table.insert(self._downUrls, newUrl)
                end
                print("#down urls", unpack(self._downUrls))

                loadVersion()
            else
                self:updateComplete()
            end
        else
            self:updateComplete()
        end

    end
    if self.config.sdkId ~= 0 then
        self._updateView:setTips("正在为您启动游戏.")
        print("#self.config.verUrl", self.config.verUrl, "url.txt")
        self.loader:getRemote(self.config.verUrl.."url.txt".."?"..tostring(math.random(1,100000)), urlHandler)
    else
        self:updateComplete()
    end

end

function GameUpdate:updateNext()
    if self._updatePaths ~= nil and #self._updatePaths > 0 then
        self._currentItem = table.remove(self._updatePaths, 1)
        -- self._updateView:setTips("策划们正策划一场更新("..self._currentItem.p..")...")
        self:setNowProgress(self.percentPast, ".")
        print("#self._currentItem", self._currentItem.v, self._currentItem.p, self._currentItem.t)

        local pVersion = self._currentItem.t .. self._currentItem.p

        local pastDownVal = self.config:readKV(self._packKey) 
        print("#pastDown", pastDownVal, pVersion)
        if pastDownVal == pVersion then
            --要下载的版本包已经在本地, 开始解压
            self.mgr:uncompress()
        else
            --local downloadFile = self.config.verUrl..self._currentItem.p..".zip?"..tostring(math.random(1,100000)) 
            -- local downloadFile = self.config.verUrl..self._currentItem.p..".zip" 

            local url = self._downUrls[1]
            if self._hasError == 1 and self._downUrls[2] ~= nil then
                url = self._downUrls[2]
            end
            self._hasError = self._hasError + 1
            local downloadFile = url..self._currentItem.p..".zip" 

            if self.mgr.setWritablePath ~= nil and self.mgr.setUncompressFilePath ~= nil then
                if self._currentItem.t == "a" or self._currentItem.t == "g" then
                    self.mgr:setWritablePath(self._writablePath)
                    self.mgr:setUncompressFilePath(self._writablePath)
                else
                    self.mgr:setWritablePath(self._versionPath)
                    self.mgr:setUncompressFilePath(self._versionPath)
                end
            end
            self.mgr:download( downloadFile )
        end
    else
        if self._updateView ~= nil then 
            self._updateView:setTips("正在为您启动游戏,请稍等...")
        end
        self:updateComplete()
        self._currentItem = nil
    end
end 

function GameUpdate:updateComplete()
	-- 更新完之后要不要重启
    self.config.downUrls = self._downUrls
	if self.isNeedRestart then
		gameRestart()
	else
		self.gameScene:enterGame()
	end
	self:dispose()
end

function GameUpdate:dispose()
	self.config = nil
	self.json = nil
	self._updateView = nil
	self.gameScene = nil
end

GameConfig = class("GameConfig", function(self,platformId)
	local self = {}
	self.verUrl = "http://login.sdbgj.m.37.com/urlconfig/"
    self.downUrls = { }
    
	self.resVersion = i100
    self.resVersion = self.resVersion or 396
	self.gmpanelTip = ""
	self.openPay = true
	self.noOpenPayTip = "充值还未开放"
    self.bigVersion = ""
    self.shenHeBigVersion = {}

	self.platformId = platformId --系统类型
    self.sdkId = 0 --平台代理Id
    --原始版本不需要加
    self.trunkType = ""
	--资源路径名称
    self.sources = "default"
    self.plusList = { "b0" }
    
    self.server_list = {}
    self.last_server = {1}
	return self
end)

function GameConfig:setSdkId(sdkId)
    self.sdkId = sdkId
    if(self.sdkId == nil)then self.sdkId = 0 end
    if self.sdkId > 1000 and self.sdkId ~= 9999999 then
        self.sources = tostring(self.sdkId)
    end
end

function GameConfig:isIOS()
	return self.platformId == cc.PLATFORM_OS_IPHONE or self.platformId == cc.PLATFORM_OS_IPAD or self.platformId == cc.PLATFORM_OS_MAC
end

function GameConfig:isAndroid()
	return self.platformId == cc.PLATFORM_OS_ANDROID
end

function GameConfig:isWin()
    return self.platformId == cc.PLATFORM_OS_WINDOWS
end

function GameConfig:inSDK()
    return self.sdkId ~= 0 and self.sdkId ~= 9999999
end

function GameConfig:writeKV(key, val)
    cc.UserDefault:getInstance():setStringForKey(key,val) 
    cc.UserDefault:getInstance():flush()
end

function GameConfig:readKV(key)
    return cc.UserDefault:getInstance():getStringForKey(key)
end

-------------------------------接口基类开始--------------------------------------------
BaseInterface = class("BaseInterface",function(self,gameScene,en,className)
    local bi = {}
    bi._en = en
    bi._className = className
    bi._gameScene = gameScene
    bi._configData = gameScene.config
    bi._callGameFunc = nil

    bi.initSdk = 6 
    bi.getBigVersion = 13 --获取大版本号

    -- 回调 java回调lua参数
    bi.initComplete_call = 107

    return bi
end)

function BaseInterface:setCallGameFunc(func)
    self._callGameFunc = func
end

--array to string
function BaseInterface:pkgParams(paramList)
    if(paramList == nil or #paramList == 0)then return "" end
    return table.concat(paramList,"#@$")
end

--string to array
function BaseInterface:parseParams(str)
    local result = {}
    local token = "#@$"
    string.gsub(str, '[^'..token..']+', function(word)
        table.insert(result, word)
    end)
    return result
end

function BaseInterface:setCallback()
    if(self._en == nil)then return nil end

    function callback(values)
        local data = values
        if(self._configData:isAndroid())then
            data = self._gameScene.json.decode(values)
        end
        local status = tonumber(data.status)
        local vals = self:parseParams(data.vals)
        if(status == self.initComplete_call)then
            print("初始化sdk完成") 
            if self._gameScene.updateView ~= nil then
                self._gameScene.updateView:setTips("正在为您启动游戏!")
            end
            self._gameScene:startLuaUpdate()
        end

        if(self._callGameFunc ~= nil)then
            self._callGameFunc(status,vals)
        end
    end
    if(self._configData:isAndroid())then
        self._en.callStaticMethod(self._className, "setCallbackF",{callback})
    elseif(self._configData:isIOS())then
        self._en.callStaticMethod(self._className, "setCallbackF",{callback = callback})
    end
end

function BaseInterface:luaCall(values)
    print("#BaseInterface:luaCall", unpack(values))
    if(self._en == nil)then return nil end
    local param = self:pkgParams(values)
    local ok,ret = false,nil
    if(self._configData:isAndroid())then
        ok,ret = self._en.callStaticMethod(self._className, "luaCall",{param})
    elseif(self._configData:isIOS())then
        ok,ret = self._en.callStaticMethod(self._className, "luaCall",{data = param})
    end
    if(ok)then
        if(self._configData:isAndroid())then
            return ret
        elseif(self._configData:isIOS())then
            return ret.data
        end
    end
    return nil
end

-------------------------------接口基类结束--------------------------------------------